c--ggpfix1.f
c--removes short gaps in files (.ggs or .ggp) by linear interpolation
c--modified from ggpscan.f and g1sfix1a.f may 2011
c----------------------------------------------------------------------------
      program GGPFIX1
c--dfport is is a portability requirement in Digital Fortran 
c--to enable file renaming
      use dfport
      implicit real*8(a-h,o-y),integer*4(i-n),logical*1(z)
      character*80 hline
      character*40 fname1,fname2,fname3
	character*30 fmt1
	character*8 ch1,c7,c9
	character*6 ch2
	character*4 cym
	character*3 cxt,ctim
	character*2 cpp
      data c7,c9/'77777777','99999999'/
c
      write(*,'(/a)') ' GGPFIX1 - fixes short gaps in ggp/ggs files'
      write(*,'(a)')  ' -------------------------------------------'
c
c--auxiliary file contains data format, and time limits
c     nprnt = print level
c	intvs = sampling interval in sec
c     ifix = 1 to fix gaps (0 just reports them)
c	cpp = file prefix (2 characters)
c--output files
	open(12,file='ggpfix1.out')
      open(9,file='ggpfix1.in')
      read(9,*) nprnt,intvs,ifix
      if(intvs.eq.1) then
	  cxt='ggs'
	  ctim='sec'
	elseif(intvs.eq.60) then
	  cxt='ggp'
	  ctim='min'
      endif
	read(9,'(a)') cpp
	read(9,'(a)') fmt1
	write(12,'(2a)') ' input format ',fmt1
c--read extreme values and length of gap in appropriate units (sec,min)
c  gl and gh are the limits for the gravity voltages to be allowed
c  any values outside this range will be treated as a 999 value (true gap)
c  pl and ph are the same for the pressure voltages
c  these have to be supplied in this auxiliary file 
	read(9,*) gl,gh,lgmax
	read(9,*) pl,ph,lpmax
	write(12,'(a,i5,2x,a)') ' max gravity gap length  = ',lgmax,ctim
	write(12,'(a,i5,2x,a)') ' max pressure gap length = ',lpmax,ctim
c--reads a series of files
c--get starting and finishing (year,month,day)
      read(9,*) iy1,im1,id1
c--get starting (year,month,day)
      read(9,*) iy2,im2,id2
c--convert to fractional year
      call ymdfy(iy2,im2,id2,23,59,59,fyear2)
      write(12,'(a,3i5,f12.6)') ' ending date ',iy2,im2,id2,fyear2
c  iy1,im1,id1 = date of first sample processed
c  iy2,im2,id2 = date of last sample processed
c  ky is truncated year (2 digits)
	iy=iy1
      ky=iy-100*(iy/100)
	km=im1
	kd=id1
      ii=1
c
c--construct input filename, assuming prefix is 'cpp', extension cxt
  10  idt=100*ky+km
      write(cym,'(i4.4)') idt
	fname1=cpp//cym//"00."//cxt
c	print *, fname1
c	pause
c--check if end of time period
	iy=1900+ky
	if(ky.lt.50) iy=iy+100
      call ymdfy(iy,km,kd,0,0,0,fyear1)
      if(fyear1.gt.fyear2) goto 100
c--does it exist
      inquire(file=fname1,exist=zexist)
	if(zexist) goto 12
      write(*,'(2a)') ' missing ',fname1
      write(12,'(2a)') ' missing ',fname1
      goto 40
c
c--read file
  12	open(1,file=fname1)
      write(12,'(/2a)') ' scanning ',fname1
      write(*,'(/2a)') ' scanning ',fname1
c--assume no output file, and open temp file 
      zopen=.false.
      open(4,file='temp')
c--read header
  20  read(1,'(a)',err=990) hline
c--until we get to the data
      if(hline(1:8).ne.c7) goto 20
	zgap=.false.
	lg=0
	ng=0
	na=0
c--read line 
  30  read(1,fmt1,end=40,err=990) ch1,ch2,gg,pp
      if(ch1.eq.c9) goto 40
c--check for gravity data outside critical values
      zg = zcrit(gg,gl,gh)
      if(.not.zgap) then
        if(zg) then
c--we have a gap
          lg=lg+1
	    ng=ng+1
	    ib=ii0
	    gb=gg0
	    pb=pp0
	    zgap=.true.
          write(*,6000) ' gap ',ng,ch1,ch2,ib,gb,pb
	    write(12,6000) ' gap ',ng,ch1,ch2,ib,gb,pb
        else
	    ii0=ii
	    gg0=gg
	    pp0=pp
        endif
      else
	  if(zg) then
	    lg=lg+1
	  else
c--end of gap, save information for short gaps
	    ie=ii
	    ge=gg
	    pe=pp
          write(*,6010) lg,ie,ge,pe
          write(12,6010) lg,ie,ge,pe
	    if(lg.le.lgmax) then
	      na=na+1
            write(4,'(4i10,4f15.6)') na,lg,ib,ie,gb,ge,pb,pe
          endif
	    lg=0
	    zgap=.false.
	  endif
      endif
6000  format(a,i5,1x,a8,1x,a6,i9,2f12.5)
6010  format(19x,i7,i9,2f12.5)
	ii=ii+1
	goto 30
c
c--end of file, rewind temp file, na=number of gaps
  40  if(ifix.eq.0) goto 45
      rewind(4)
      if(na.eq.0) then
	  write(*,*) ' no gaps to be fixed'
	  goto 45
	endif
c--test if the .\old directory for fixed files exists
      istat=chdir("old")
      if(istat.eq.0) then
	  istat1=chdir("..\")	!yes, change back
	else
	  write(*,*) ' directory .\old does not exist, quitting .. '
	  goto 999
	endif
c--rewind the input file
      rewind(1)
c--open fixed file in same directory
	fname2=fname1(1:8)//".fx1"
	open(2,file=fname2)
	write(*,*) ' new file ',fname2
      read(4,*) ngap,lg,ib,ie,gb,ge,pb,pe
      zopen=.true.
      ii=1
  50  read(1,'(a)',err=990) hline
      write(2,'(a)') trim(hline)
      if(hline(1:8).ne.c7) goto 50
c
c--read line 
  60  read(1,fmt1,end=40,err=990) ch1,ch2,gg,pp
c--write original values according to size
      call wrtgp(gg,pp,ch1,ch2,fmt1)
	ii=ii+1
	if(ii.le.ib) goto 60
c--start of gap, get slope
      slopeg=(ge-gb)/dble(ie-ib)
	slopep=(pe-pb)/dble(ie-ib)
c--read next lg lines and interpolate
c--replace gg,pp with interpolated values
      do ll=1,lg
        read(1,fmt1,end=40,err=990) ch1,ch2,gg,pp
        gg=gb+(ii-ib)*slopeg 
        pp=pb+(ii-ib)*slopep 
	  write(2,fmt1) ch1,ch2,gg,pp
        ii=ii+1
      enddo
	ii=ie
      write(*,'(a,i5,a,i5)') ' gap ',ngap,' fixed, length = ',lg
      write(12,'(a,i5,a,i5)') ' gap ',ngap,' fixed, length = ',lg
c--if another gap, read it and repeat
      if(ngap.lt.na) then
        read(4,*) ngap,lg,ib,ie,gb,ge,pb,pe
	  goto 60
      endif
      write(12,'(i7,a)') na,' gaps fixed '		  	  		  	  	       	  	      
c--finished gaps, complete file
  65  read(1,fmt1,end=40,err=990) ch1,ch2,gg,pp
      if(ch1.eq.c9) then
        write(2,fmt1) c9
	  goto 45
      endif
c--continue writing 
      call wrtgp(gg,pp,ch1,ch2,fmt1)  
	ii=ii+1
	goto 65

c--new month
  45  ii=1
      km=km+1
c--change year?
	if(km.gt.12) then
        iy=iy+1
        ky=iy-100*(iy/100)
	  km=1
      endif
	kd=1
	close(1)
	close(2)
c--if we opened a fixed file, do some renaming
	if(zopen) then
c--rename original file in "old" as *.ggx
	  fname3="old\"//fname1(1:8)//".ggx"
        istat=rename(fname1,fname3)
	  if(istat.ne.0) then
	    write(*,*) ' could not rename ',fname1
	    goto 995
	  endif
c--rename the fixed file *.fx1 as *.ggp
	  fname3=fname2(1:8)//"."//cxt
        istat=rename(fname2,fname3)
	  if(istat.ne.0) then
	    write(*,*) ' could not rename ',fname2
	    goto 995
	  endif
      endif
c--delete the gap file
	close(4,status='delete')
	goto 10
c
 100  write(*,*) ' done'
      stop
c
c--errors
 990  write(*,*) 'error reading ',fname1,' last line read: '
      write(*,'(a)') hline
 995  close(4,status='delete')
 999  stop
      end
c--------------------------------------------------------------------------
      subroutine wrtgp(gg,pp,ch1,ch2,fmt1)
c
c--this subroutine ensures that gravity and pressure 
c--are written according to correct format
c
	implicit real*8(a-h,o-y)
	character*30 fmt1,fmt2,fmt3,fmt4
	character*8 ch1
	character*6 ch2
      data g99,p99/2*99999.999d0/,dh/9999.d0/,
     &     fmt2/'(a8,1x,a6,f10.6,f10.3)'/,
     &     fmt3/'(a8,1x,a6,f10.3,f10.4)'/,
     &     fmt4/'(a8,1x,a6,2f10.3)'/
c
	if(gg.lt.dh) then
	  if(pp.lt.dh) write(2,fmt1) ch1,ch2,gg,pp
        if(pp.gt.dh) write(2,fmt2) ch1,ch2,gg,p99
	else
	  if(pp.lt.dh) write(2,fmt3) ch1,ch2,g99,pp
        if(pp.gt.dh) write(2,fmt4) ch1,ch2,g99,p99
      endif
c
      return
      end
c------------------------------------------------------------------------
      function zcrit(y,y1,y2)
c
c logical function: true if y is outside range (y1,y2)
c
      implicit real*8(a-h,o-y),logical*1(z)
      zcrit=.false.
      if(y.lt.y1.or.y.gt.y2) zcrit=.true.
      return
      end
