c--ggpscan.f
c--scans a ggp file (sec or 1 min) for bad time increments
c--modified from tfscan.f may 2011
c----------------------------------------------------------------------------
      program GGPSCAN
      implicit real*8(a-h,o-y),integer*4(i-n),logical*1(z)
      character*80 hline
      character*40 fname1
	character*30 fmt1
	character*8 ch1,c7,c9
	character*6 ch2
	character*4 cym
	character*3 cxt
	character*2 cpp
c
      common /misc/gg,pp,fyear,intvs,nprnt
      data c7,c9/'77777777','99999999'/
c
      write(*,'(/a)') ' GGPSCAN - scans files for bad time increments'
      write(*,'(a)')  ' --------------------------------------------'
c
c--auxiliary file contains data format, and time limits
c	ntype	= 1 is for a series of files
c			= 2 is for a single file
c	intvs = sampling interval in sec
c	cpp = file prefix (2 characters)
c--output files
	open(12,file='ggpscan.out')
      open(9,file='ggpscan.in')
      read(9,*) nprnt,intvs,ntype
      if(intvs.eq.1) then
	  cxt='ggs'
	elseif(intvs.eq.60) then
	  cxt='ggp'
      endif
      if(ntype.eq.2) then
        read(9,'(a)') fname1
        ii=1	   
	  goto 12
      endif
	read(9,'(a)') cpp
	read(9,'(a)') fmt1
	write(*,'(a)') fmt1
c--reads a series of files
c--get starting and finishing (year,month,day)
      read(9,*) iy1,im1,id1
c--get starting (year,month,day)
      read(9,*) iy2,im2,id2
c--convert to fractional year
      call ymdfy(iy2,im2,id2,23,59,59,fyear2)
      write(12,'(a,3i5,f12.6)')
     & 	'beginning day ',iy1,im1,id1,fyear1, 
     & 	'ending day    ',iy2,im2,id2,fyear2
c--set the flags
c  nfa = index for monthly file
c  nf  = index for daily file
c  iy1,im1,id1 = date of first sample processed
c  iy2,im2,id2 = date of last sample processed
c  ky is truncated year (2 digits)
      nfa=1
	nf=1
	iy=iy1
      ky=iy-100*(iy/100)
	km=im1
	kd=id1
      ii=1
c
c--construct input filename, assuming prefix is 'cpp', extension cxt
  10  idt=100*ky+km
      write(cym,'(i4.4)') idt
	fname1=cpp//cym//"00."//cxt
c--check if end of time period
	iy=1900+ky
	if(ky.lt.50) iy=iy+100
      call ymdfy(iy,km,kd,0,0,0,fyear1)
      if(fyear1.gt.fyear2) goto 100
c--does it exist
      inquire(file=fname1,exist=zexist)
	if(zexist) goto 12
      write(*,'(2a)') ' missing ',fname1
      write(12,'(2a)') ' missing ',fname1
      goto 45
c
  12	open(1,file=fname1)
      write(12,'(2a)') ' reading ',fname1
      write(*,'(2a)') ' reading ',fname1
c
c--read header
  20  read(1,'(a)',err=990) hline
c--until we get to the data
      if(hline(1:8).ne.c7) goto 20
c
c--data lines
  30  read(1,'(a)',end=40,err=990) hline
c      write(*,'(a)') hline
	ch1=hline(1:8)
	ch2=hline(10:16)
      if(ch1.eq.c9) goto 40
c--convert date and time to integers
      read(ch1,'(i4,2i2)') iy,mo,id
	read(ch2,'(3i2)') ih,mn,is
c--get fractional year
      call ymdfy(iy,mo,id,ih,mn,is,fyear)
c--check sequence
      call chkseq(ii,iy,mo,id,ih,mn,is,kk)
      ii=ii+1
      goto 30
c
c--end of file
  40  nt=ii-1
      nd=nt*intvs/86400
      write(12,6040) ' number samples ',nt,' days ',nd
6040  format(15x,a,i9,a,i3)
	if(kk.gt.0) then
	  write(*,'(i7,a)') kk,' bad steps '
        write(12,'(i7,a)') kk,' bad steps '
      endif
  45  if(ntype.eq.2) goto 100
c--new month
      ii=1
      km=km+1
c--change year?
	if(km.gt.12) then
        iy=iy+1
        ky=iy-100*(iy/100)
	  km=1
      endif
	kd=1
	goto 10
c
 100  write(*,*) ' done'
      stop
c
c--errors
 990  write(*,*) 'error reading ',fname1,' last line read: '
      write(*,'(a)') hline
 999  stop
      end
c-----------------------------------------------------------
      subroutine chkseq(ii,iy,mo,id,ih,mn,is,kk)
c
c  check the time sequencing of a file in Tsoft or preterna format
c  inputs: ii = current line number, which
c               left unchanged if the time sample is correct
c               updated if values are inserted by the subroutine
c          iy,mo,id,ih,mn,is = date and time of sample
c  depending how a date/time is out of sequence the following occurs
c  date/time negative: reject (return 1)
c            duplicate of last: reject
c            positive but 
c               < 2 x intvs: correct input date/time exit normally
c               > 2 x intvs: introduce gap values and reset times
c     
      implicit real*8(a-h,o-z),integer*4(i-n)
      common /misc/gg,pp,fyear,intvs,nprnt
c
c--convert date to mean julian days
	if(nprnt.gt.1) write(*,*) ' ii = ',ii
      call julian(iy,mo,id,ih,mn,is,rmjd)
c
c--first pass
      if(ii.eq.1) then
	  intv=intvs
        kk=0
	  goto 50
      endif
c--get increment in seconds
	intv=nint(86400.d0*(rmjd-rmjd0))
c--subsequent passes, flag bad steps
      if(intv.ne.intvs) then
	  step=dble(intv-intvs)/dble(intvs)
	  nv1=nint(step)
        kk=kk+1
        write(12,6010) ' bad step ',iy,mo,id,ih,mn,is,step 
      endif
6010  format(5x,a,i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,i2,g13.5) 
c
  50  rmjd0=rmjd
      return
	end
