c--tfscan.f
c--scans a tsf file (sec or 1 min) for bad time increments
c--modified from tsf2ggpm.f may 2011
c----------------------------------------------------------------------------
      program TFSCAN
      implicit real*8(a-h,o-y),integer*4(i-n),logical*1(z)
      character*80 hline
      character*40 fname1
	character*30 fmt1,cdir
	character*6 ch2
	character*4 cxt
	character*2 cpp
c
      write(*,'(/a)') ' TFSCAN - scans files for bad time increments'
      write(*,'(a)')  ' ---------------------------------------------'
c
c--auxiliary file contains data format, and time limits
      open(9,file='tfscan.in')
c--nprnt is the print control, intvs is the time sampling in sec
c--cpp is the input file prefix (2 character)
c--cxt is the TSF file extension (3 character)
c--cdir is the directory for the input files relative to current directory
c--fmt1 is the input format for the (date,time,grav,press)
      read(9,*) nprnt,intvs
      read(9,'(a2,a5)') cpp,cxt
	read(9,'(a)') cdir
c--len_trim character length without trailing zeros
	lcd=len_trim(cdir)
c	write(*,*) cdir(1:lcd)
	read(9,'(a)') fmt1
c--output files
	open(12,file='tfscan.out')
c--reads a series of files
c--get starting and finishing (year,month,day)
      read(9,*) iy1,im1,id1
c--get starting (year,month,day)
      read(9,*) iy2,im2,id2
c--convert to fractional year
      call ymdfy(iy2,im2,id2,0,0,0,fyear2)
      write(12,'(a,3i5,f12.6)')
     & 	'beginning day ',iy1,im1,id1,fyear1, 
     & 	'ending day    ',iy2,im2,id2,fyear2
c--set the flags
c  nfa = index for monthly file
c  nf  = index for daily file
c  iy1,im1,id1 = date of first sample processed
c  iy2,im2,id2 = date of last sample processed
c  ky is truncated year (2 digits)
      nfa=1
	nf=1
	iy=iy1
      ky=iy-100*(iy/100)
	km=im1
	kd=id1
      ii=1
	jd=1
	nsd=86400/intvs
c
c--check if end of time period
  10  iy=1900+ky
	if(ky.lt.50) iy=iy+100
      call ymdfy(iy,km,kd,0,0,0,fyear1)
c      write(*,*) fyear1,fyear2
      if(fyear1.ge.fyear2) goto 100
c--construct input filename, assuming prefix is 'cpp', extension '.cxt'
      idt=100*(100*ky+km)+kd
	print *,ky,km,kd,idt 
	write(ch2,'(i6.6)') idt
      fname1=cdir(1:lcd)//cpp//ch2//cxt
c--does it exist
      inquire(file=fname1,exist=zexist)
	if(zexist) goto 12
	if(nprnt.gt.0) then
        write(*,'(2a)') 'missing ',fname1
        write(12,'(2a)') 'missing ',fname1
      endif
	goto 40
  12	open(1,file=fname1)
      kk=0
c
c--read tsf file
  20  read(1,'(a)',err=990) hline
c--until we get to the data
      if(hline(1:6).ne.'[DATA]') goto 20
c
c--data lines
  30  read(1,fmt1,end=40,err=990) iy,mo,id,ih,mn,is,gg,pp
c--get fractional year
      call ymdfy(iy,mo,id,ih,mn,is,fyear)
c--check sequence
      call chkseq(nprnt,intvs,ii,iy,mo,id,ih,mn,is,kk,fname1)
      ii=ii+1
	jd=jd+1
c	jm=jm+1
c	pause
      goto 30
c
c--end of day file
  40  ns=jd-1
  45  kd=kd+1
      nf=nf+1
	nfa=nfa+1
	jd=1
      call ndaym(iy,km,nday)
	if(kd.le.nday) goto 10
      jm=1
      km=km+1
c--change year?
	if(km.gt.12) then
        iy=iy+1
        ky=iy-100*(iy/100)
	  km=1
      endif
	kd=1
	goto 10
c
 100  write(*,*) ' done'
      stop
c
c--errors
 990  write(*,*) 'error reading ',fname1,' last line read: '
      write(*,fmt1) iy,mo,id,mn,is
 999  stop
      end
c-----------------------------------------------------------
      subroutine chkseq(nprnt,intvs,ii,iy,mo,id,ih,mn,is,kk,fname1)
c
c  check the time sequencing of a file in Tsoft or preterna format
c  inputs: ii = current line number, which
c               left unchanged if the time sample is correct
c               updated if values are inserted by the subroutine
c          iy,mo,id,ih,mn,is = date and time of sample
c          kk = index for how many bad steps this file
c  depending how a date/time is out of sequence the following occurs
c  date/time negative: reject (return 1)
c            duplicate of last: reject
c            positive but 
c               < 2 x intvs: correct input date/time exit normally
c               > 2 x intvs: introduce gap values and reset times
c     
      implicit real*8(a-h,o-z),integer*4(i-n)
      character*40 fname1
c
c--convert date to mean julian days
	if(nprnt.gt.1) write(*,*) ' ii = ',ii
      call julian(iy,mo,id,ih,mn,is,rmjd)
c--first pass
      if(ii.eq.1) then
	  intv=intvs
	  goto 50
      endif
c--get increment in seconds
	intv=nint(86400.d0*(rmjd-rmjd0))
c--subsequent passes, flag bad steps
      if(intv.ne.intvs) then
	  if(kk.eq.0) then
          write(12,'(2a)') 'file ',fname1
          write(*,'(2a)') 'file ',fname1
        endif
	  step=dble(intv-intvs)/dble(intvs)
	  nv1=nint(step)+1
        kk=kk+1
        write(*,6010) ' bad step ',iy,mo,id,ih,mn,is,step
        write(12,6010) ' bad step ',iy,mo,id,ih,mn,is,step
      endif
6010  format(5x,a,i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,i2,g13.5) 
c
  50  rmjd0=rmjd
      return
	end
