c--tsf2ggp.f
c--converts .tsf files (sec or 1 min) to ggp format by month
c--modified from tsf2ggpm.f may 2011, feb 2012
c----------------------------------------------------------------------------
      program TSF2GGP
      implicit real*8(a-h,o-y),integer*4(i-n),logical*1(z)
      character*80 hline,cstar
      character*40 hfile,fname1,fname2
	character*30 fmt1,cdir
	character*8 ch1,c7,c9
	character*6 ch2
	character*4 cym
	character*3 cxt
	character*2 cdd,cpp
      common /misc/gg,pp,gcal,pcal,fyear,rjda,rjdb,intvs,nprnt,mi,zfout
c
c--files opened:
c	unit 9 = auxiliary file "tsf2ggp.aux"
c     if nout=1 we have these 2 special files (unit 11 needs the calibrations)
c		unit 10 = single ggp file, all months "ap-all.ggp", ggp
c		unit 11 = single file "ap-all.fy" fractional yr,g,p
c	unit 12 = general output "tsf2ggp.out"
c	unit 1 = input daily [.tsf] files
c	unit 2 = output monthly [.ggp] files
c	unit 3 = ggp header file
c
      data c7,c9/'77777777','99999999'/,
     &	g99,p99/2*99999.999d0/,g0/0.0/,dh/9999.d0/
c
      write(*,'(/a)') ' TSF2GGP - converts sec or min TSF to GGP'
      write(*,'(a)')  ' corrects timing errors and adds gaps     '
      write(*,'(a)')  ' ----------------------------------------'
c
c--auxiliary file contains
c	nprnt = print level
c	intvs = default increment (s)
c	nout  = 1 write combined output files 
c             WARNING THIS MAY BE A LARGE FILE FOR SECOND DATA!
c	cpp   = station code (a2)
c	hfile = station header filename
c	fmt1  = data format for tsf file
      open(9,file='tsf2ggp.in')
      read(9,*) nprnt,intvs,nout
      read(9,'(a)') cpp
	read(9,'(a)') cdir
	lcd=len_trim(cdir)
	read(9,'(a)') hfile
	read(9,'(a)') fmt1
c--file extension depends on sec or min
      if(intvs.eq.1) then
	  cxt='ggs'
	elseif(intvs.eq.60) then
	  cxt='ggp'
      endif
c--output files
      zfout=.false.
      if(nout.eq.1) then
	  open(10,file='all-data.'//cxt)
	  open(11,file='all-data.fy')
        zfout=.true.
      endif
	open(12,file='tsf2ggp.out')
c  iy1,im1,id1 = date of first sample required
c  iy2,im2,id2 = date of last sample required
      read(9,*) iy1,im1,id1
      read(9,*) iy2,im2,id2
c--convert ending date to fractional year
      call ymdfy(iy2,im2,id2,23,59,59,fyear2)
      write(12,'(a,3i5,f16.10)') ' ending date ',iy2,im2,id2,fyear2
c
c--initialize the time variables
c  nfa = index for monthly file
c  nf  = index for daily file
c  ky is truncated year (2 digits)
c  mi is monthly index
      nfa=1
	nf=1
	iy=iy1
      ky=iy-100*(iy/100)
	km=im1
	kd=id1
      mi=1
c
c--construct input filename, convert integers to characters
  10  idt=100*ky+km
      write(cym,'(i4.4)') idt
      write(cdd,'(i2.2)') kd
	fname1=cdir(1:lcd)//cpp//cym//cdd//".046"
c--first file of month, get output name
c  zero = true until there is data for the month
	if(nf.eq.1) then
        fname2='ap'//cym//"00."//cxt
        write(12,'(2a)') ' output file is ',fname2
        write(*,'(2a)') ' output ',fname2
        open(2,file=fname2)
        zero=.true.
c--write header at start of month
c--first header line, insert filename
        open(3,file=hfile)
        jj=1
  15    read(3,'(a)') cstar
        lc=len_trim(cstar)
c--adjust filename on first line
        if(jj.eq.1) cstar(25:34)=cym//"00."//cxt
	  write(2,'(a)') cstar(1:lc)
	  if(zfout.and.nfa.eq.1) write(10,'(a)') cstar(1:lc)
c--extract calibration factors (assuming header is in GGP required format!!)
        if(cstar(1:7).eq.'Gravity') read(cstar(22:31),'(f10.4)') gcal
        if(cstar(1:8).eq.'Pressure') read(cstar(22:31),'(f10.4)') pcal
c--next header line
	  jj=jj+1
	  if(cstar(1:5).ne.'C****') goto 15
	  write(2,17) c7,g0,g0
	  if(zfout.and.nfa.eq.1) write(10,17) c7,g0,g0
  17    format(a,7x,f10.6,f10.4)
	  close(3)
c	  write(*,'(a,2f10.4)') ' calibrations ',gcal,pcal
c--find start/end days and number of samples this month
	  iy=1900+ky
	  if(ky.lt.50) iy=iy+100
        call julian(iy,km,1,0,0,0,rjda)
        call ndaym(iy,km,nday)
        call julian(iy,km,nday,23,59,59,rjdb)
	  write(12,'(a,2f15.6)') ' month limits ',rjda,rjdb
	  nsm=nday*86400/intvs
        if(nprnt.gt.0) then
	    write(12,'(a,2i7)') ' year and month ',iy,km
	    write(12,'(a,i7)') ' default interval ',intvs
	    write(12,'(a,i7)') ' # days this month = ',nday
          write(12,'(a,i7)') ' # samples this month = ',nsm
	  endif
	endif
c--check if end of time period
      call ymdfy(iy,km,kd,0,0,0,fyear1)
      if(fyear1.gt.fyear2) goto 100
c--does this day file exist
      inquire(file=fname1,exist=zexist)
	if(.not.zexist) then
        write(*,'(2a)')  '    missing ',fname1
	  write(12,'(2a)') '    missing ',fname1
c--get next file
        goto 40
      endif
c
c--start reading
  12	open(1,file=fname1)
      zero=.false.
      write(12,'(2a)') '   reading ',fname1
      write(*,'(2a)')  '   reading ',fname1
  20  read(1,'(a)',err=990) hline
c--until we get to the data
      if(hline(1:6).ne.'[DATA]') goto 20
c
c--data lines
  30  read(1,fmt1,end=40,err=990) iy,mo,id,ih,mn,is,gg,pp
c--check sequence and recompute date and time
      call chkseq(iy,mo,id,ih,mn,is,*30)
c--convert integers to character strings
	write(ch1,'(i4,2i2.2)') iy,mo,id
	write(ch2,'(3i2.2)') ih,mn,is
	call julian(iy,mo,id,ih,mn,is,rmjd)
c--write to monthly ggp file, replacing *** values
      if(gg.lt.dh) then
	  if(pp.lt.dh) write(2,6001) ch1,ch2,gg,pp
	  if(pp.gt.dh) write(2,6002) ch1,ch2,gg,p99
      else
        if(pp.lt.dh) write(2,6003) ch1,ch2,g99,pp
        if(pp.gt.dh) write(2,6004) ch1,ch2,g99,p99
      endif
6001  format(a8,1x,a6,f10.6,f10.4)
6002  format(a8,1x,a6,f10.6,f10.3)
6003  format(a8,1x,a6,f10.3,f10.4)
6004  format(a8,1x,a6,2f10.3)
c--write to all-ggp file and fyear file
      if(zfout) then
        call ymdfy(iy,mo,id,ih,mn,is,fyear)  
        write(10,6010) ch1,ch2,gg,pp
        write(11,6020) fyear,gg*gcal,pp*pcal
6010    format(a8,1x,a6,f10.6,f10.4)
6020    format(f13.7,2f11.4)
      endif
      mi=mi+1
      goto 30
c
c--next day file
  40  kd=kd+1
      nf=nf+1
	nfa=nfa+1
c--save date as julian day
	call juldat(iy,mo,id,ih,mn,is,rmjd)
	if(kd.le.nday) goto 10
c
c--fill missing samples at end of month file
      nsi=mi-1
	if(nsi.lt.nsm) then 
	  nv1=nsm-nsi+1
        write(*,*) nv1,' missing values at end of month'
        write(12,*) nv1,' missing values at end of month'
        call julian(iy,mo,id,ih,mn,is,rmjd)	
        write(*,6030) ' trailing',iy,mo,id,ih,mn,is,rmjd,intvs
        write(12,6030) ' trailing',iy,mo,id,ih,mn,is,rmjd,intvs                     
	  call fill99(nv1,rmjd)
      endif
c
c--end this month file
      write(2,'(a)') c9
      close(2)
      nod=nsi*intvs/86400
      write(12,'(a,i7)') ' number of samples    = ',nsi
      write(12,'(a,i7)') ' number of full days  = ',nod
      write(12,'(a)')  ' ----------------------------'
c
c--new month
      mi=1
      km=km+1
c--change year?
	if(km.gt.12) then
        iy=iy+1
        ky=iy-100*(iy/100)
	  km=1
      endif
      nf=1
	kd=1
      goto 10
6030  format(a,i5,5i3,f15.6,i9) 
c
c--finish
 100  if(zfout) write(10,'(a)') c9
      if(zero) then
	  print *, ' ... no data, delete this file ' 
        close(2, status='delete')
      endif
      print *, ' done'
      stop
c
c--errors
 990  print *, ' error reading ',fname1,' last line read: '
      write(*,fmt1) iy,mo,id,ih,mn,is,gg,pp
 999  stop
      end
c-------------------------------------------------------------------
      subroutine chkseq(iy,mo,id,ih,mn,is,*)
c
c  check the time sequencing of a file in Tsoft or preterna format
c    inputs:
c		iy,mo,id,ih,mn,is = date and time of sample
c	depending how a date/time is out of sequence the following occurs
c	intv <= 0, quit with return 1 (no action)
c           = default, save rmjd and exit             
c           = approx 1 step, correct date/time
c           > 1 x step, introduce gap values and reset dates/times
c     
      implicit real*8(a-h,o-y),integer*4(i-n),logical*1(z)
      common /misc/gg,pp,gcal,pcal,fyear,rjda,rjdb,intvs,nprnt,mi,zfout
c
c--convert date to mean julian days, omit dates not in this month
      call julian(iy,mo,id,ih,mn,is,rmjd)
      if(rmjd.lt.rjda.or.rmjd.gt.rjdb) return 1
c
c--start of month, see if missing samples
      if(mi.eq.1) then
	  intv=intvs
        drmd=dble(intvs)/86400.d0
c--number of missing samples at start of month
        rmj=(rmjd-rjda)*86400.d0
        mjs=nint(rmj)
        write(12,6000) ' start ',iy,mo,id,ih,mn,is,rmjd,intv
c--fill missing values at start with 999s
        if(mjs.gt.0) then
          step=rmj/dble(intvs)
	    nval=nint(step)
	    nv1=nval+1
	    rmjd=rjda-drmd  
		if(nprnt.gt.0) 
     &    write(12,6000) ' leading',iy,mo,id,ih,mn,is,rmjd,intv
          write(*,6000) ' leading',iy,mo,id,ih,mn,is,rmjd,intvs           
	    call fill99(nv1,rmjd)
        endif             
	  goto 50
      endif
c
c--check interval, if good exit normally
c--skip a time duplicate or negative increment
	intv=nint(86400.d0*(rmjd-rmjd0))
	if(intv.eq.intvs) then
	  goto 50
      elseif(intv.le.0) then
	  if(nprnt.gt.0) 
     &	  write(12,6000) ' <=zero ',iy,mo,id,ih,mn,is,rmjd,intv
        return 1
      endif
c--interval larger than default, find # values to insert
      step=dble(intv-intvs)/dble(intvs)
	nval=nint(step)
	nv1=nval+1
c--if only 1 step add appropriate fraction of a day
c--convert to date
      if(nv1.eq.1) then
        if(nprnt.gt.0) 
     &    write(12,6000) ' step   ',iy,mo,id,ih,mn,is,rmjd,intv
	  rmjd=rmjd0+drmd
        call juldat(iy,mo,id,ih,mn,is,rmjd)
        if(nprnt.gt.0)
     &    write(12,6000) ' replace',iy,mo,id,ih,mn,is,rmjd,intv  
        goto 50
c--more than 1 step insert nv1 999 values
      elseif(nv1.gt.1) then
        rmjd=rmjd0
        if(nprnt.gt.0)
     &    write(12,6000) ' fill   ',iy,mo,id,ih,mn,is,rmjd,nv1
        call fill99(nv1,rmjd)
        call juldat(iy,mo,id,ih,mn,is,rmjd)
        if(nprnt.gt.0) write(12,6000) 
     &      ' endgap ',iy,mo,id,ih,mn,is,rmjd,intv
        goto 50
	endif
6000  format(a,i5,5i3,f15.6,i9) 
6010  format(a8,1x,a6,1x,f10.5,f10.4)
c
c--normal exit
  50  rmjd0=rmjd
      return
	end
c---------------------------------------------------------
      subroutine fill99(nv1,rmjd)
c
c--fills gaps with nv1 x 999 values for grav and pressure
c
      implicit real*8(a-h,o-y),integer*4(i-n),logical*1(z)
	character*8 ch1t,ch1
	character*6 ch2t,ch2
c
      common /misc/gg,pp,gcal,pcal,fyear,rjda,rjdb,intvs,nprnt,mi,zfout
c
      data g99,p99/2*99999.999d0/
	drmd=dble(intvs)/86400.d0
c
c--increment julian day and convert to date/time
      do jj=1,nv1
	  intj=jj*intvs
	  rmjd=rmjd+drmd
        call juldat(iy,mo,id,ih,mn,is,rmjd)
c--convert integers to character strings
	  write(ch1t,'(i4,2i2.2)') iy,mo,id
	  write(ch2t,'(3i2.2)') ih,mn,is
c--write to files all but last value
        if(jj.lt.nv1) then
	    if(nprnt.gt.1) 
     &       write(12,6000) ' 9999 ',iy,mo,id,ih,mn,is,rmjd,intj
          write(2,6010) ch1t,ch2t,g99,p99 
c--output for large single files
          if(zfout) then
            call ymdfy(iy,mo,id,ih,mn,is,fyear)
c--change integer dates to characters and write to file
	      write(ch1,'(i4,2i2.2)') iy,mo,id
	      write(ch2,'(3i2.2)') ih,mn,is		  		    
            write(10,6010) ch1,ch2,g99,p99
            write(11,6020) fyear,g99,p99
          endif
          mi=mi+1
	  endif 
      enddo
6000  format(10x,a,i4,5i3,f15.6,i9) 
6010  format(a8,1x,a6,2f10.3)
6020  format(f13.7,2f11.3)
c
      return
	end
